/*
 *******************************************************
  
 cat and mice benchmark, wodes08, libfaudes 2.09

 this file provides models for individual maze levels of the cat-and-mouse
 tower described in the benchmark example. In each level, the number of 
 cats/mice that are present in each room is described. Hence, in this model,
 it is not envisaged to distinguish individual cats and mice. 
  
 *******************************************************
 */ 



#include <stdio.h>
#include <iostream>
#include "libfaudes.h"
#include "catsMice_levels_models.h"
   
using namespace std;

 // Maze(rMaze, rMazeSpec, n, k, l, rHighAlphabet)
void Maze(cGenerator& rMaze, cGenerator& rMazeSpec, const int n, const int k, const int l, EventSet& rHighAlphabet)
{
      FD_DF("Maze(" << rMaze.Name() << "," << rMazeSpec.Name() << "," << n << ", " << k << ", " << l << "," << rHighAlphabet.Name() << ")");
      // prepare the data structures for the algorithm
      rHighAlphabet.Clear();
      rMaze.Clear();
      rMazeSpec.Clear();
      // catch invalid inputs
      if(n<1)
      {
          FD_DF("Cat:: the minimum value for n is 1: empty result.");
          return;
      }
    
      if(k<0)
      {
          FD_DF("Cat::  the minimum value for k is 1: empty result.");
          return;
      }
      std::string lstr = ToStringInteger(l);
      Idx currentState, nextState, event;
      Idx catState, mouseState, nextCatState, nextMouseState;
      // ====================================================================
      // Computation of the plant model and the local specification of room 0
      // ====================================================================
      
      // Model of the cats in room 0
      // helper data structures
      cGenerator catRoom0, mouseRoom0, catCounterRoom0, mouseCounterRoom0, tmp, specTmp;
      // Create a state representing the empty room. Initially the room is empty, except for the case l == 1.
      currentState = catRoom0.InsState("C0,0("+lstr+")");
      if(l != 1){
        catRoom0.SetInitState(currentState);
        catRoom0.SetMarkedState(currentState);    
      }
      // Insert states for all possible numbers of cats present in the room (up to k) and
      // add transitions for arrival and departure of a cat. 
      for( int j = 1; j <= k; j++){
        // events for transitions inside the maze level
        nextState = catRoom0.InsState("C0,"+ToStringInteger(j)+"("+lstr+")");
        event = catRoom0.InsControllableEvent("c4-0("+lstr+")");
        catRoom0.SetTransition(currentState,event,nextState); 
            
        event = catRoom0.InsControllableEvent("c2-0("+lstr+")");
        catRoom0.SetTransition(currentState,event,nextState); 
            
        event = catRoom0.InsControllableEvent("c0-1("+lstr+")");
        catRoom0.SetTransition(nextState,event,currentState);
    
        event = catRoom0.InsControllableEvent("c0-3("+lstr+")");
        catRoom0.SetTransition(nextState,event,currentState);
        // events for transitions between maze levels   
        // consider the levels 1,6,...
        if(l % 5 == 1 && n > l){
          event = catRoom0.InsControllableEvent("c"+ToStringInteger(l+1)+"-"+lstr );
          catRoom0.SetTransition(currentState,event,nextState);  
          rHighAlphabet.Insert(event);
                       
          event = catRoom0.InsControllableEvent("c"+lstr+"-"+ToStringInteger(l+1) );
          catRoom0.SetTransition(nextState,event,currentState); 
          rHighAlphabet.Insert(event);  
        }                                                   
        // consider the levels 2,7,...
        if(l % 5 == 2){
          event = catRoom0.InsControllableEvent("c"+ToStringInteger(l-1)+"-"+lstr );
          catRoom0.SetTransition(currentState,event,nextState);  
          rHighAlphabet.Insert(event);
                       
          event = catRoom0.InsControllableEvent("c"+lstr+"-"+ToStringInteger(l-1) );
          catRoom0.SetTransition(nextState,event,currentState);         
          rHighAlphabet.Insert(event); 
        }
        currentState = nextState;
      }
      // in the first maze level, the initial state and marked state represents room 0 filled with cats.
      if(l== 1){
        catRoom0.SetInitState(currentState);
        catRoom0.SetMarkedState(currentState);         
      }
      #ifdef FAUDES_DEBUG_WODES08
            catRoom0.Write("./data/catRoom0.gen");
      #endif
            
      // Model of the mice in room 0
      // Insert a state representing the empty room 0. Initially, room 0 does not contain any mice. 
      currentState = mouseRoom0.InsState("M0,0("+lstr+")");
      mouseRoom0.SetInitState(currentState);
      mouseRoom0.SetMarkedState(currentState);    
      // Insert states for all possible numbers of mice in room 0, and add the transitions for 
      // arrival and departure of mice.
      for( int j = 1; j <= k; j++){
        // transitions inside the current maze level
        nextState = mouseRoom0.InsState("M0,"+ToStringInteger(j)+"("+lstr+")");
        event = mouseRoom0.InsControllableEvent("m1-0("+lstr+")");
        mouseRoom0.SetTransition(currentState,event,nextState); 
            
        event = mouseRoom0.InsControllableEvent("m3-0("+lstr+")");
        mouseRoom0.SetTransition(currentState,event,nextState); 
            
        event = mouseRoom0.InsControllableEvent("m0-2("+lstr+")");
        mouseRoom0.SetTransition(nextState,event,currentState);
    
        event = mouseRoom0.InsControllableEvent("m0-4("+lstr+")");
        mouseRoom0.SetTransition(nextState,event,currentState);   
        // transitions between maze levels
        // consider the levels 1,6,...
        if(l % 5 == 1 && n > l){
          event = mouseRoom0.InsControllableEvent("m"+ToStringInteger(l+1)+"-"+lstr );
          mouseRoom0.SetTransition(currentState,event,nextState);  
          rHighAlphabet.Insert(event);
                       
          event = mouseRoom0.InsControllableEvent("m"+lstr+"-"+ToStringInteger(l+1) );
          mouseRoom0.SetTransition(nextState,event,currentState);
          rHighAlphabet.Insert(event);   
        }                                                   
        // consider the levels 2,7,...
        if(l % 5 == 2){
          event = mouseRoom0.InsControllableEvent("m"+ToStringInteger(l-1)+"-"+lstr );
          mouseRoom0.SetTransition(currentState,event,nextState);  
          rHighAlphabet.Insert(event);
                       
          event = mouseRoom0.InsControllableEvent("m"+lstr+"-"+ToStringInteger(l-1) );
          mouseRoom0.SetTransition(nextState,event,currentState);  
          rHighAlphabet.Insert(event);        
        }
        currentState = nextState;
      }
      #ifdef FAUDES_DEBUG_WODES08
            mouseRoom0.Write("./data/mouseRoom0.gen");
      #endif
    
      // specification for cats and mice in room 0. It has to be ensured that the presence of cats and mice
      // in room 0 is mutually exclusive. 
      cGenerator specRoom0;
      // Initially, room 0 is empty except for level 1
      catState = specRoom0.InsState("0,0|0,0("+lstr+")");
      mouseState = catState;
      if(l != 1){
        specRoom0.SetInitState(catState);
        specRoom0.SetMarkedState(catState);    
      }
      // the specification counts the number of cats/mice present in room 0 and makes sure that 
      // whenever at least one cat is present, no mouse can enter and vice versa. 
      for( int j = 1; j <= k; j++){
        // transitions inside the maze lavel for cats
        nextCatState = specRoom0.InsState("0,"+ToStringInteger(j)+"|0,0("+lstr+")");
        event = specRoom0.InsControllableEvent("c4-0("+lstr+")");
        specRoom0.SetTransition(catState,event,nextCatState); 
            
        event = specRoom0.InsControllableEvent("c2-0("+lstr+")");
        specRoom0.SetTransition(catState,event,nextCatState); 
            
        event = specRoom0.InsControllableEvent("c0-1("+lstr+")");
        specRoom0.SetTransition(nextCatState,event,catState);
    
        event = specRoom0.InsControllableEvent("c0-3("+lstr+")");
        specRoom0.SetTransition(nextCatState,event,catState); 
        // transitions between maze levels for cats
        // consider the levels 1,6,...
        if(l % 5 == 1 && n > l){
          event = specRoom0.InsControllableEvent("c"+ToStringInteger(l+1)+"-"+lstr );
          specRoom0.SetTransition(catState,event,nextCatState);  
                       
          event = specRoom0.InsControllableEvent("c"+lstr+"-"+ToStringInteger(l+1) );
          specRoom0.SetTransition(nextCatState,event,catState);   
        }                                                   
        // consider the levels 2,7,...
        if(l % 5 == 2){
          event = specRoom0.InsControllableEvent("c"+ToStringInteger(l-1)+"-"+lstr );
          specRoom0.SetTransition(catState,event,nextCatState);  
                       
          event = specRoom0.InsControllableEvent("c"+lstr+"-"+ToStringInteger(l-1) );
          specRoom0.SetTransition(nextCatState,event,catState);          
        }
        catState = nextCatState;
        // transitions inside the maze level for mice
        nextMouseState = specRoom0.InsState("0,0|0,"+ToStringInteger(j)+"("+lstr+")");
                            
        event = specRoom0.InsControllableEvent("m1-0("+lstr+")");
        specRoom0.SetTransition(mouseState,event,nextMouseState); 
            
        event = specRoom0.InsControllableEvent("m3-0("+lstr+")");
        specRoom0.SetTransition(mouseState,event,nextMouseState); 
            
        event = specRoom0.InsControllableEvent("m0-2("+lstr+")");
        specRoom0.SetTransition(nextMouseState,event,mouseState);
    
        event = specRoom0.InsControllableEvent("m0-4("+lstr+")");
        specRoom0.SetTransition(nextMouseState,event,mouseState);  
        // transitions between maze levels for the mice 
        // consider the levels 1,6,...
        if(l % 5 == 1 && n > l){
          event = specRoom0.InsControllableEvent("m"+ToStringInteger(l+1)+"-"+lstr );
          specRoom0.SetTransition(mouseState,event,nextMouseState);  
                       
          event = specRoom0.InsControllableEvent("m"+lstr+"-"+ToStringInteger(l+1) );
          specRoom0.SetTransition(nextMouseState,event,mouseState);   
        }                                                   
        // consider the levels 2,7,...
        if(l % 5 == 2){
          event = specRoom0.InsControllableEvent("m"+ToStringInteger(l-1)+"-"+lstr );
          specRoom0.SetTransition(mouseState,event,nextMouseState);  
                       
          event = specRoom0.InsControllableEvent("m"+lstr+"-"+ToStringInteger(l-1) );
          specRoom0.SetTransition(nextMouseState,event,mouseState);          
        }
        mouseState = nextMouseState;
      }
      if(l== 1){
        specRoom0.SetInitState(catState);
        specRoom0.SetMarkedState(catState);         
      }
      #ifdef FAUDES_DEBUG_WODES08
            specRoom0.Write("./data/specRoom0.gen");    
      #endif
        
      // ====================================================================
      // Computation of the plant model and the local specification of room 1
      // ====================================================================
      
      // The operations as for room 0 are repeated. Refer to a more detailed description 
      // in the above construction. 
      // helper data structures
      cGenerator catRoom1, mouseRoom1, catCounterRoom1, mouseCounterRoom1;
      currentState = catRoom1.InsState("C1,0("+lstr+")");
        
      catRoom1.SetInitState(currentState);
      catRoom1.SetMarkedState(currentState);    
      // model for room 1: k cats (initial state is empty)
      for( int j = 1; j <= k; j++){
        // buffer model for the cats
        nextState = catRoom1.InsState("C1,"+ToStringInteger(j)+"("+lstr+")");
        event = catRoom1.InsControllableEvent("c0-1("+lstr+")");
        catRoom1.SetTransition(currentState,event,nextState); 
            
        event = catRoom1.InsUncontrollableEvent("c3-1("+lstr+")");
        catRoom1.SetTransition(currentState,event,nextState); 
            
        event = catRoom1.InsUncontrollableEvent("c1-3("+lstr+")");
        catRoom1.SetTransition(nextState,event,currentState);
    
        event = catRoom1.InsControllableEvent("c1-2("+lstr+")");
        catRoom1.SetTransition(nextState,event,currentState);   
        // consider the levels 2,7,...
        if(l % 5 == 2 && l < n){
          event = catRoom1.InsControllableEvent("c"+ToStringInteger(l+1)+"-"+lstr );
          catRoom1.SetTransition(currentState,event,nextState);  
          rHighAlphabet.Insert(event);
                       
          event = catRoom1.InsControllableEvent("c"+lstr+"-"+ToStringInteger(l+1) );
          catRoom1.SetTransition(nextState,event,currentState); 
          rHighAlphabet.Insert(event);  
        }                                                   
        // consider the levels 3,8,...
        if(l % 5 == 3){
          event = catRoom1.InsControllableEvent("c"+ToStringInteger(l-1)+"-"+lstr );
          catRoom1.SetTransition(currentState,event,nextState); 
          rHighAlphabet.Insert(event); 
                       
          event = catRoom1.InsControllableEvent("c"+lstr+"-"+ToStringInteger(l-1) );
          catRoom1.SetTransition(nextState,event,currentState); 
          rHighAlphabet.Insert(event);         
        }
        currentState = nextState;
      }
      #ifdef FAUDES_DEBUG_WODES08
            catRoom1.Write("./data/catRoom1.gen");
      #endif
    
      // mouse in room 1
      currentState = mouseRoom1.InsState("M1,0("+lstr+")");
      mouseRoom1.SetInitState(currentState);
      mouseRoom1.SetMarkedState(currentState);    
      // model for room 1: k mice (initial state is empty
      for( int j = 1; j <= k; j++){
        // buffer model for the mice
        nextState = mouseRoom1.InsState("M1,"+ToStringInteger(j)+"("+lstr+")");
        event = mouseRoom1.InsControllableEvent("m2-1("+lstr+")");
        mouseRoom1.SetTransition(currentState,event,nextState); 
            
        event = mouseRoom1.InsControllableEvent("m1-0("+lstr+")");
        mouseRoom1.SetTransition(nextState,event,currentState);
    
        // consider the levels 2,6,...
        if(l % 5 == 2 && l < n){
          event = mouseRoom1.InsControllableEvent("m"+ToStringInteger(l+1)+"-"+lstr );
          mouseRoom1.SetTransition(currentState,event,nextState);  
          rHighAlphabet.Insert(event);
                       
          event = mouseRoom1.InsControllableEvent("m"+lstr+"-"+ToStringInteger(l+1) );
          mouseRoom1.SetTransition(nextState,event,currentState);   
          rHighAlphabet.Insert(event);
        }                                                   
        // consider the levels 3,8,...
        if(l % 5 == 3){
          event = mouseRoom1.InsControllableEvent("m"+ToStringInteger(l-1)+"-"+lstr );
          mouseRoom1.SetTransition(currentState,event,nextState);  
          rHighAlphabet.Insert(event);
                       
          event = mouseRoom1.InsControllableEvent("m"+lstr+"-"+ToStringInteger(l-1) );
          mouseRoom1.SetTransition(nextState,event,currentState);    
          rHighAlphabet.Insert(event);      
        }
        currentState = nextState;
      }
    
      #ifdef FAUDES_DEBUG_WODES08
            mouseRoom1.Write("./data/mouseRoom1.gen");
      #endif
    
      // specification for cat and mouse in room 1
      cGenerator specRoom1;
        
      catState = specRoom1.InsState("1,0|1,0("+lstr+")");
      mouseState = catState;
        
      specRoom1.SetInitState(catState);
      specRoom1.SetMarkedState(catState);    
      for( int j = 1; j <= k; j++){
        // two disjoint buffer models for cat and mice with only common empty state
        nextCatState = specRoom1.InsState("1,"+ToStringInteger(j)+"|1,0("+lstr+")");
        event = specRoom1.InsControllableEvent("c0-1("+lstr+")");
        specRoom1.SetTransition(catState,event,nextCatState); 
            
        event = specRoom1.InsUncontrollableEvent("c3-1("+lstr+")");
        specRoom1.SetTransition(catState,event,nextCatState); 
            
        event = specRoom1.InsUncontrollableEvent("c1-3("+lstr+")");
        specRoom1.SetTransition(nextCatState,event,catState);
    
        event = specRoom1.InsControllableEvent("c1-2("+lstr+")");
        specRoom1.SetTransition(nextCatState,event,catState);   
        // consider the levels 2,7,...
        if(l % 5 == 2 && l < n){
          event = specRoom1.InsControllableEvent("c"+ToStringInteger(l+1)+"-"+lstr );
          specRoom1.SetTransition(catState,event,nextCatState);  
                       
          event = specRoom1.InsControllableEvent("c"+lstr+"-"+ToStringInteger(l+1) );
          specRoom1.SetTransition(nextCatState,event,catState);   
        }                                                   
        // consider the levels 3,8,...
        if(l % 5 == 3){
          event = specRoom1.InsControllableEvent("c"+ToStringInteger(l-1)+"-"+lstr );
          specRoom1.SetTransition(catState,event,nextCatState);  
                       
          event = specRoom1.InsControllableEvent("c"+lstr+"-"+ToStringInteger(l-1) );
          specRoom1.SetTransition(nextCatState,event,catState);          
        }
        catState = nextCatState;
        // mouse buffer
        nextMouseState = specRoom1.InsState("1,0|1,"+ToStringInteger(j)+"("+lstr+")");
                            
        event = specRoom1.InsControllableEvent("m2-1("+lstr+")");
        specRoom1.SetTransition(mouseState,event,nextMouseState); 
            
        event = specRoom1.InsControllableEvent("m1-0("+lstr+")");
        specRoom1.SetTransition(nextMouseState,event,mouseState);
              
        // consider the levels 2,7,...
        if(l % 5 == 2 && l < n){
          event = specRoom1.InsControllableEvent("m"+ToStringInteger(l+1)+"-"+lstr );
          specRoom1.SetTransition(mouseState,event,nextMouseState);  
                       
          event = specRoom1.InsControllableEvent("m"+lstr+"-"+ToStringInteger(l+1) );
          specRoom1.SetTransition(nextMouseState,event,mouseState);   
        }                                                   
        // consider the levels 3,8,...
        if(l % 5 == 3){
          event = specRoom1.InsControllableEvent("m"+ToStringInteger(l-1)+"-"+lstr );
          specRoom1.SetTransition(mouseState,event,nextMouseState);  
                       
          event = specRoom1.InsControllableEvent("m"+lstr+"-"+ToStringInteger(l-1) );
          specRoom1.SetTransition(nextMouseState,event,mouseState);          
        }
        mouseState = nextMouseState;
      }
      #ifdef FAUDES_DEBUG_WODES08
            specRoom1.Write("./data/specRoom1.gen");  
      #endif
       
      // ====================================================================
      // Computation of the plant model and the local specification of room 2
      // ====================================================================
      
      // The operations as for room 0 are repeated. Refer to a more detailed description 
      // in the above construction. 
      // helper data structures
      cGenerator catRoom2, mouseRoom2, catCounterRoom2, mouseCounterRoom2;
      currentState = catRoom2.InsState("C2,0("+lstr+")");
        
      catRoom2.SetInitState(currentState);
      catRoom2.SetMarkedState(currentState);    
      // model for room 2: k cats (initial state is empty)
      for( int j = 1; j <= k; j++){
        // buffer model for the cats
        nextState = catRoom2.InsState("C2,"+ToStringInteger(j)+"("+lstr+")");
        event = catRoom2.InsControllableEvent("c1-2("+lstr+")");
        catRoom2.SetTransition(currentState,event,nextState); 
    
        event = catRoom2.InsControllableEvent("c2-0("+lstr+")");
        catRoom2.SetTransition(nextState,event,currentState);   
        // consider the levels 3,8,...
        if(l % 5 == 3 && l < n){
          event = catRoom2.InsControllableEvent("c"+ToStringInteger(l+1)+"-"+lstr );
          catRoom2.SetTransition(currentState,event,nextState);  
          rHighAlphabet.Insert(event);
                       
          event = catRoom2.InsControllableEvent("c"+lstr+"-"+ToStringInteger(l+1) );
          catRoom2.SetTransition(nextState,event,currentState);   
          rHighAlphabet.Insert(event);
        }                                                   
        // consider the levels 4,9,...
        if(l % 5 == 4){
          event = catRoom2.InsControllableEvent("c"+ToStringInteger(l-1)+"-"+lstr );
          catRoom2.SetTransition(currentState,event,nextState);  
          rHighAlphabet.Insert(event);
                       
          event = catRoom2.InsControllableEvent("c"+lstr+"-"+ToStringInteger(l-1) );
          catRoom2.SetTransition(nextState,event,currentState); 
          rHighAlphabet.Insert(event);         
        }
        currentState = nextState;
      } 
    
      #ifdef FAUDES_DEBUG_WODES08
            catRoom2.Write("./data/catRoom2.gen");
      #endif
      
      // mice in room 2
      currentState = mouseRoom2.InsState("M2,0("+lstr+")");
      mouseRoom2.SetInitState(currentState);
      mouseRoom2.SetMarkedState(currentState);    
      // model for room 2: k mice (initial state is empty)
      for( int j = 1; j <= k; j++){
        // buffer model for the mice
        nextState = mouseRoom2.InsState("M2,"+ToStringInteger(j)+"("+lstr+")");
        event = mouseRoom2.InsControllableEvent("m0-2("+lstr+")");
        mouseRoom2.SetTransition(currentState,event,nextState); 
            
        event = mouseRoom2.InsControllableEvent("m2-1("+lstr+")");
        mouseRoom2.SetTransition(nextState,event,currentState);
    
        // consider the levels 3,8,...
        if(l % 5 == 3 && l < n){
          event = mouseRoom2.InsControllableEvent("m"+ToStringInteger(l+1)+"-"+lstr );
          mouseRoom2.SetTransition(currentState,event,nextState);  
          rHighAlphabet.Insert(event);
                       
          event = mouseRoom2.InsControllableEvent("m"+lstr+"-"+ToStringInteger(l+1) );
          mouseRoom2.SetTransition(nextState,event,currentState);   
          rHighAlphabet.Insert(event);
        }                                                   
        // consider the levels 4,9,...
        if(l % 5 == 4){
          event = mouseRoom2.InsControllableEvent("m"+ToStringInteger(l-1)+"-"+lstr );
          mouseRoom2.SetTransition(currentState,event,nextState);  
          rHighAlphabet.Insert(event);
                       
          event = mouseRoom2.InsControllableEvent("m"+lstr+"-"+ToStringInteger(l-1) );
          mouseRoom2.SetTransition(nextState,event,currentState);   
          rHighAlphabet.Insert(event);       
        }
        currentState = nextState;
      }
    
      #ifdef FAUDES_DEBUG_WODES08
            mouseRoom2.Write("./data/mouseRoom2.gen");
      #endif
      
      // specification for cats and mice in room 2
      cGenerator specRoom2;
        
      catState = specRoom2.InsState("2,0|2,0("+lstr+")");
      mouseState = catState;
        
      specRoom2.SetInitState(catState);
      specRoom2.SetMarkedState(catState);    

      for( int j = 1; j <= k; j++){
        // two disjoint buffer models for cat and mice with only common empty state
        nextCatState = specRoom2.InsState("2,"+ToStringInteger(j)+"|2,0("+lstr+")");
        event = specRoom2.InsControllableEvent("c1-2("+lstr+")");
        specRoom2.SetTransition(catState,event,nextCatState); 
    
        event = specRoom2.InsControllableEvent("c2-0("+lstr+")");
        specRoom2.SetTransition(nextCatState,event,catState);   
        // consider the levels 3,8,...
        if(l % 5 == 3 && l < n){
          event = specRoom2.InsControllableEvent("c"+ToStringInteger(l+1)+"-"+lstr );
          specRoom2.SetTransition(catState,event,nextCatState);  
                       
          event = specRoom2.InsControllableEvent("c"+lstr+"-"+ToStringInteger(l+1) );
          specRoom2.SetTransition(nextCatState,event,catState);   
        }                                                   
        // consider the levels 4,9,...
        if(l % 5 == 4){
          event = specRoom2.InsControllableEvent("c"+ToStringInteger(l-1)+"-"+lstr );
          specRoom2.SetTransition(catState,event,nextCatState);  
                       
          event = specRoom2.InsControllableEvent("c"+lstr+"-"+ToStringInteger(l-1) );
          specRoom2.SetTransition(nextCatState,event,catState);          
        }
        catState = nextCatState;
        // mouse buffer
        nextMouseState = specRoom2.InsState("2,0|2,"+ToStringInteger(j)+"("+lstr+")");
                            
        event = specRoom2.InsControllableEvent("m0-2("+lstr+")");
        specRoom2.SetTransition(mouseState,event,nextMouseState); 
            
        event = specRoom2.InsControllableEvent("m2-1("+lstr+")");
        specRoom2.SetTransition(nextMouseState,event,mouseState);
              
        // consider the levels 3,8,...
        if(l % 5 == 3 && l < n){
          event = specRoom2.InsControllableEvent("m"+ToStringInteger(l+1)+"-"+lstr );
          specRoom2.SetTransition(mouseState,event,nextMouseState);  
                       
          event = specRoom2.InsControllableEvent("m"+lstr+"-"+ToStringInteger(l+1) );
          specRoom2.SetTransition(nextMouseState,event,mouseState);   
        }                                                   
        // consider the levels 4,9,...
        if(l % 5 == 4){
          event = specRoom2.InsControllableEvent("m"+ToStringInteger(l-1)+"-"+lstr );
          specRoom2.SetTransition(mouseState,event,nextMouseState);  
                       
          event = specRoom2.InsControllableEvent("m"+lstr+"-"+ToStringInteger(l-1) );
          specRoom2.SetTransition(nextMouseState,event,mouseState);          
        }
        mouseState = nextMouseState;
      }
        
      #ifdef FAUDES_DEBUG_WODES08
            specRoom2.Write("./data/specRoom2.gen");   
      #endif
        
      // ====================================================================
      // Computation of the plant model and the local specification of room 3
      // ====================================================================
      
      // The operations as for room 0 are repeated. Refer to a more detailed description 
      // in the above construction. 
      // helper data structures
      cGenerator catRoom3, mouseRoom3, catCounterRoom3, mouseCounterRoom3;
      currentState = catRoom3.InsState("C3,0("+lstr+")");
        
      catRoom3.SetInitState(currentState);
      catRoom3.SetMarkedState(currentState);    
      // model for room 3: k cats (initial state is empty)
      for( int j = 1; j <= k; j++){
        // buffer model for the cats
        nextState = catRoom3.InsState("C3,"+ToStringInteger(j)+"("+lstr+")");
        event = catRoom3.InsControllableEvent("c0-3("+lstr+")");
        catRoom3.SetTransition(currentState,event,nextState); 
            
        event = catRoom3.InsUncontrollableEvent("c1-3("+lstr+")");
        catRoom3.SetTransition(currentState,event,nextState); 
    
        event = catRoom3.InsControllableEvent("c3-4("+lstr+")");
        catRoom3.SetTransition(nextState,event,currentState);   
    
        event = catRoom3.InsUncontrollableEvent("c3-1("+lstr+")");
        catRoom3.SetTransition(nextState,event,currentState); 
        // consider the levels 4,9,...
        if(l % 5 == 4 && l < n){
          event = catRoom3.InsControllableEvent("c"+ToStringInteger(l+1)+"-"+lstr );
          catRoom3.SetTransition(currentState,event,nextState);  
          rHighAlphabet.Insert(event);
                       
          event = catRoom3.InsControllableEvent("c"+lstr+"-"+ToStringInteger(l+1) );
          catRoom3.SetTransition(nextState,event,currentState);   
          rHighAlphabet.Insert(event);
        }                                                   
        // consider the levels 5,10,...
        if(l % 5 == 0){
          event = catRoom3.InsControllableEvent("c"+ToStringInteger(l-1)+"-"+lstr );
          catRoom3.SetTransition(currentState,event,nextState); 
          rHighAlphabet.Insert(event); 
                       
          event = catRoom3.InsControllableEvent("c"+lstr+"-"+ToStringInteger(l-1) );
          catRoom3.SetTransition(nextState,event,currentState);    
          rHighAlphabet.Insert(event);      
        }
        currentState = nextState;
      }
    
      #ifdef FAUDES_DEBUG_WODES08
            catRoom3.Write("./data/catRoom3.gen");
      #endif
      
      // mice in room 3
      currentState = mouseRoom3.InsState("M3,0("+lstr+")");
      mouseRoom3.SetInitState(currentState);
      mouseRoom3.SetMarkedState(currentState);    
      // model for room 3: k mice (initial state empty)
      for( int j = 1; j <= k; j++){
        // buffer model for the mice
        nextState = mouseRoom3.InsState("M3,"+ToStringInteger(j)+"("+lstr+")");
        event = mouseRoom3.InsControllableEvent("m4-3("+lstr+")");
        mouseRoom3.SetTransition(currentState,event,nextState); 
            
        event = mouseRoom3.InsControllableEvent("m3-0("+lstr+")");
        mouseRoom3.SetTransition(nextState,event,currentState);
    
        // consider the levels 4,9,...
        if(l % 5 == 4 && l < n){
          event = mouseRoom3.InsControllableEvent("m"+ToStringInteger(l+1)+"-"+lstr );
          mouseRoom3.SetTransition(currentState,event,nextState);  
          rHighAlphabet.Insert(event);
                       
          event = mouseRoom3.InsControllableEvent("m"+lstr+"-"+ToStringInteger(l+1) );
          mouseRoom3.SetTransition(nextState,event,currentState);   
          rHighAlphabet.Insert(event);
        }                                                   
        // consider the levels 5,10,...
        if(l % 5 == 0){
          event = mouseRoom3.InsControllableEvent("m"+ToStringInteger(l-1)+"-"+lstr );
          mouseRoom3.SetTransition(currentState,event,nextState);  
          rHighAlphabet.Insert(event);
                       
          event = mouseRoom3.InsControllableEvent("m"+lstr+"-"+ToStringInteger(l-1) );
          mouseRoom3.SetTransition(nextState,event,currentState);  
          rHighAlphabet.Insert(event);        
        }
        currentState = nextState;
      }
    
      #ifdef FAUDES_DEBUG_WODES08
            mouseRoom3.Write("./data/mouseRoom3.gen");
      #endif
      
      // specification for cats and mice in room 3
      cGenerator specRoom3;
        
      catState = specRoom3.InsState("3,0|3,0("+lstr+")");
      mouseState = catState;
        
      specRoom3.SetInitState(catState);
      specRoom3.SetMarkedState(catState);    

      for( int j = 1; j <= k; j++){
        // two disjoint buffer models for cat and mice with only common empty state
        nextCatState = specRoom3.InsState("3,"+ToStringInteger(j)+"|3,0("+lstr+")");
        event = specRoom3.InsControllableEvent("c0-3("+lstr+")");
        specRoom3.SetTransition(catState,event,nextCatState); 
        
        event = specRoom3.InsUncontrollableEvent("c1-3("+lstr+")");
        specRoom3.SetTransition(catState,event,nextCatState); 
    
        event = specRoom3.InsControllableEvent("c3-4("+lstr+")");
        specRoom3.SetTransition(nextCatState,event,catState);   
    
        event = specRoom3.InsUncontrollableEvent("c3-1("+lstr+")");
        specRoom3.SetTransition(nextCatState,event,catState); 
        // consider the levels 4,9,...
        if(l % 5 == 4 && l < n){
          event = specRoom3.InsControllableEvent("c"+ToStringInteger(l+1)+"-"+lstr );
          specRoom3.SetTransition(catState,event,nextCatState);  
                       
          event = specRoom3.InsControllableEvent("c"+lstr+"-"+ToStringInteger(l+1) );
          specRoom3.SetTransition(nextCatState,event,catState);   
        }                                                   
        // consider the levels 5,10,...
        if(l % 5 == 0){
          event = specRoom3.InsControllableEvent("c"+ToStringInteger(l-1)+"-"+lstr );
          specRoom3.SetTransition(catState,event,nextCatState);  
                       
          event = specRoom3.InsControllableEvent("c"+lstr+"-"+ToStringInteger(l-1) );
          specRoom3.SetTransition(nextCatState,event,catState);          
        }
        catState = nextCatState;
        // mouse buffer
        nextMouseState = specRoom3.InsState("3,0|3,"+ToStringInteger(j)+"("+lstr+")");
                            
        event = specRoom3.InsControllableEvent("m4-3("+lstr+")");
        specRoom3.SetTransition(mouseState,event,nextMouseState); 
            
        event = specRoom3.InsControllableEvent("m3-0("+lstr+")");
        specRoom3.SetTransition(nextMouseState,event,mouseState);
              
        // consider the levels 4,9,...
        if(l % 5 == 4 && l < n){
          event = specRoom3.InsControllableEvent("m"+ToStringInteger(l+1)+"-"+lstr );
          specRoom3.SetTransition(mouseState,event,nextMouseState);  
                       
          event = specRoom3.InsControllableEvent("m"+lstr+"-"+ToStringInteger(l+1) );
          specRoom3.SetTransition(nextMouseState,event,mouseState);   
        }                                                   
        // consider the levels 5,10,...
        if(l % 5 == 0){
          event = specRoom3.InsControllableEvent("m"+ToStringInteger(l-1)+"-"+lstr );
          specRoom3.SetTransition(mouseState,event,nextMouseState);  
                       
          event = specRoom3.InsControllableEvent("m"+lstr+"-"+ToStringInteger(l-1) );
          specRoom3.SetTransition(nextMouseState,event,mouseState);          
        }
        mouseState = nextMouseState;
      }
    
      #ifdef FAUDES_DEBUG_WODES08
            specRoom3.Write("./data/specRoom3.gen");   
      #endif
      
      // ====================================================================
      // Computation of the plant model and the local specification of room 4
      // ====================================================================
      
      // The operations as for room 0 are repeated. Refer to a more detailed description 
      // in the above construction. 
      // helper data structures
      cGenerator catRoom4, mouseRoom4, catCounterRoom4, mouseCounterRoom4;
      currentState = catRoom4.InsState("C4,0("+lstr+")");
        
      catRoom4.SetInitState(currentState);
      catRoom4.SetMarkedState(currentState);    
      // model for room 4: k cats (initial state is empty)
      for( int j = 1; j <= k; j++){
        // buffer model for the cats
        nextState = catRoom4.InsState("C4,"+ToStringInteger(j)+"("+lstr+")");
        event = catRoom4.InsControllableEvent("c3-4("+lstr+")");
        catRoom4.SetTransition(currentState,event,nextState); 
    
        event = catRoom4.InsControllableEvent("c4-0("+lstr+")");
        catRoom4.SetTransition(nextState,event,currentState);   
    
        // consider the levels 5,10,...
        if(l % 5 == 0 && l < n){
          event = catRoom4.InsControllableEvent("c"+ToStringInteger(l+1)+"-"+lstr );
          catRoom4.SetTransition(currentState,event,nextState);  
          rHighAlphabet.Insert(event);
                       
          event = catRoom4.InsControllableEvent("c"+lstr+"-"+ToStringInteger(l+1) );
          catRoom4.SetTransition(nextState,event,currentState);   
          rHighAlphabet.Insert(event);
        }                                                   
        // consider the levels 5,10,...
        if(l % 5 == 1 && l > 1){
          event = catRoom4.InsControllableEvent("c"+ToStringInteger(l-1)+"-"+lstr );
          catRoom4.SetTransition(currentState,event,nextState);  
          rHighAlphabet.Insert(event);
                       
          event = catRoom4.InsControllableEvent("c"+lstr+"-"+ToStringInteger(l-1) );
          catRoom4.SetTransition(nextState,event,currentState);    
          rHighAlphabet.Insert(event);      
        }
        currentState = nextState;
      } 
    
      #ifdef FAUDES_DEBUG_WODES08
            catRoom4.Write("./data/catRoom4.gen");
      #endif
      
      // mice in room 4
      // all mice are initially in room 4 in the maze level n
      currentState = mouseRoom4.InsState("M4,0("+lstr+")");
      if(l != n){
        mouseRoom4.SetInitState(currentState);
        mouseRoom4.SetMarkedState(currentState);    
      }
      // model for room 4: k mice (initial state is full if level n, empty otherwise)
      for( int j = 1; j <= k; j++){
        // buffer model for the mice
        nextState = mouseRoom4.InsState("M4,"+ToStringInteger(j)+"("+lstr+")");
        event = mouseRoom4.InsControllableEvent("m0-4("+lstr+")");
        mouseRoom4.SetTransition(currentState,event,nextState); 
            
        event = mouseRoom4.InsControllableEvent("m4-3("+lstr+")");
        mouseRoom4.SetTransition(nextState,event,currentState);
    
        // consider the levels 5,10,...
        if(l % 5 == 0 && l < n){
          event = mouseRoom4.InsControllableEvent("m"+ToStringInteger(l+1)+"-"+lstr );
          mouseRoom4.SetTransition(currentState,event,nextState);  
          rHighAlphabet.Insert(event);
                       
          event = mouseRoom4.InsControllableEvent("m"+lstr+"-"+ToStringInteger(l+1) );
          mouseRoom4.SetTransition(nextState,event,currentState);
          rHighAlphabet.Insert(event);   
        }                                                   
        // consider the levels 6,11,...
        if(l % 5 == 1 && l > 1){
          event = mouseRoom4.InsControllableEvent("m"+ToStringInteger(l-1)+"-"+lstr );
          mouseRoom4.SetTransition(currentState,event,nextState); 
          rHighAlphabet.Insert(event); 
                       
          event = mouseRoom4.InsControllableEvent("m"+lstr+"-"+ToStringInteger(l-1) );
          mouseRoom4.SetTransition(nextState,event,currentState);   
          rHighAlphabet.Insert(event);       
        }
        currentState = nextState;
      }
      if(l == n){
        mouseRoom4.SetInitState(currentState);
        mouseRoom4.SetMarkedState(currentState);    
      }
    
      #ifdef FAUDES_DEBUG_WODES08
            mouseRoom4.Write("./data/mouseRoom4.gen");
      #endif
      
      // specification for cats and mice in room 4
      cGenerator specRoom4;
        
      catState = specRoom4.InsState("4,0|4,0("+lstr+")");
      mouseState = catState;
    
      if(l != n){
        specRoom4.SetInitState(catState);
        specRoom4.SetMarkedState(catState); 
      }   

      for( int j = 1; j <= k; j++){
        // two disjoint buffer models for cat and mice with only common empty state
        nextCatState = specRoom4.InsState("4,"+ToStringInteger(j)+"|4,0("+lstr+")");
        event = specRoom4.InsControllableEvent("c3-4("+lstr+")");
        specRoom4.SetTransition(catState,event,nextCatState); 
    
        event = specRoom4.InsControllableEvent("c4-0("+lstr+")");
        specRoom4.SetTransition(nextCatState,event,catState);   
    
        // consider the levels 5,10,...
        if(l % 5 == 0 && l < n){
          event = specRoom4.InsControllableEvent("c"+ToStringInteger(l+1)+"-"+lstr );
          specRoom4.SetTransition(catState,event,nextCatState);  
                       
          event = specRoom4.InsControllableEvent("c"+lstr+"-"+ToStringInteger(l+1) );
          specRoom4.SetTransition(nextCatState,event,catState);   
        }                                                   
        // consider the levels 6,11,...
        if(l % 5 == 1 && l > 1){
          event = specRoom4.InsControllableEvent("c"+ToStringInteger(l-1)+"-"+lstr );
          specRoom4.SetTransition(catState,event,nextCatState);  
                       
          event = specRoom4.InsControllableEvent("c"+lstr+"-"+ToStringInteger(l-1) );
          specRoom4.SetTransition(nextCatState,event,catState);          
        }
        catState = nextCatState;
        // mouse buffer
        nextMouseState = specRoom4.InsState("4,0|4,"+ToStringInteger(j)+"("+lstr+")");
                            
        event = specRoom4.InsControllableEvent("m0-4("+lstr+")");
        specRoom4.SetTransition(mouseState,event,nextMouseState); 
            
        event = specRoom4.InsControllableEvent("m4-3("+lstr+")");
        specRoom4.SetTransition(nextMouseState,event,mouseState);
              
        // consider the levels 5,10,...
        if(l % 5 == 0 && l < n){
          event = specRoom4.InsControllableEvent("m"+ToStringInteger(l+1)+"-"+lstr );
          specRoom4.SetTransition(mouseState,event,nextMouseState);  
                       
          event = specRoom4.InsControllableEvent("m"+lstr+"-"+ToStringInteger(l+1) );
          specRoom4.SetTransition(nextMouseState,event,mouseState);   
        }                                                   
        // consider the levels 6,11,...
        if(l % 5 == 1 && l > 1){
          event = specRoom4.InsControllableEvent("m"+ToStringInteger(l-1)+"-"+lstr );
          specRoom4.SetTransition(mouseState,event,nextMouseState);  
                       
          event = specRoom4.InsControllableEvent("m"+lstr+"-"+ToStringInteger(l-1) );
          specRoom4.SetTransition(nextMouseState,event,mouseState);          
        }
        mouseState = nextMouseState;
      }
      if(l == n){
        specRoom4.SetInitState(mouseState);
        specRoom4.SetMarkedState(mouseState); 
      }  
    
      #ifdef FAUDES_DEBUG_WODES08
            specRoom4.Write("./data/specRoom4.gen");
      #endif
      
      // The cat counter is introduced in each level, to ensure that not more than k cats can 
      // be present. This modeling step is particularly useful for the modular synthesis, as 
      // the modular models do not include this restriction, yet (i.e., more cats and mice can arrive
      // from other levels)
      cGenerator catCounter, catCounter1, catCounter2;
      Idx currentState1, currentState2, nextState1, nextState2;
      currentState1 = catCounter1.InsState("CC1("+lstr+")");
      currentState2 = catCounter2.InsState("CC2("+lstr+")");
      catCounter1.SetInitState(currentState1);
      catCounter1.SetMarkedState(currentState1);   
      catCounter2.SetInitState(currentState2);
      catCounter2.SetMarkedState(currentState2); 
      // It is considered that all cats are initially in the lowest maze level. Hence,
      // if a cat enters another level, it first has to come from below. 
      for( int j = 1; j <= k; j++){
        // buffer model for the cats ceoming from the lower level
        nextState1 = catCounter1.InsState("CC1"+ToStringInteger(j)+"("+lstr+")");
        nextState2 = catCounter2.InsState("CC2"+ToStringInteger(j)+"("+lstr+")");
        // all intermediate levels are considered. It is recorded when cats come from 
        // below, pass to the higher level, and come back.
        if(l < n && l > 1){
          event = catCounter1.InsControllableEvent("c"+ToStringInteger(l-1)+"-"+lstr );
          catCounter1.SetTransition(currentState1,event,nextState1);   
                       
          event = catCounter1.InsControllableEvent("c"+lstr+"-"+ToStringInteger(l-1) );
          catCounter1.SetTransition(nextState1,event,currentState1);   
                       
          event = catCounter2.InsControllableEvent("c"+lstr+"-"+ToStringInteger(l+1) );
          catCounter2.SetTransition(currentState2,event,nextState2);  
                            
          event = catCounter2.InsControllableEvent("c"+ToStringInteger(l+1)+"-"+lstr );
          catCounter2.SetTransition(nextState2,event,currentState2);  
        }  
        // in the lowest level, it is recorded when cats leave the level and reenter
        if(l==1 && l != n){
          if( n > 1){
    	event = catCounter1.InsControllableEvent("c"+ToStringInteger(l+1)+"-"+lstr );
    	catCounter1.SetTransition(nextState1,event,currentState1);  
                           
    	event = catCounter1.InsControllableEvent("c"+lstr+"-"+ToStringInteger(l+1) );
    	catCounter1.SetTransition(currentState1,event,nextState1);                 
          }
          else{
    	break;
          }                  
        }
        // in the highest level, it is recorded when cats enter from below and leave to the lower level                                               
        if(l == n){
          if( l > 1){
        	event = catCounter1.InsControllableEvent("c"+ToStringInteger(l-1)+"-"+lstr );
        	catCounter1.SetTransition(currentState1,event,nextState1);  
                               
        	event = catCounter1.InsControllableEvent("c"+lstr+"-"+ToStringInteger(l-1) );
        	catCounter1.SetTransition(nextState1,event,currentState1);    
          }
          else{
    	break;                      
          }      
        }
        currentState1 = nextState1;
        currentState2 = nextState2;
      }
      if( l != n && l != 1){
        cParallel(catCounter1,catCounter2,catCounter);   
      }
      else
        catCounter = catCounter1;
        
      #ifdef FAUDES_DEBUG_WODES08
          catCounter.Write("./data/catCounter.gen");
      #endif

      // mouse counter for current level. The same considerations as for the cat counter are made, considering
      // the fact that initially all mice are in the highest level. 
      cGenerator mouseCounter, mouseCounter1, mouseCounter2;
      currentState1 = mouseCounter1.InsState("MC1("+lstr+")");
      currentState2 = mouseCounter2.InsState("MC2("+lstr+")");
      mouseCounter1.SetInitState(currentState1);
      mouseCounter1.SetMarkedState(currentState1);   
      mouseCounter2.SetInitState(currentState2);
      mouseCounter2.SetMarkedState(currentState2); 
      for( int j = 1; j <= k; j++){
        // buffer model for the mice coming from higher level
        nextState1 = mouseCounter1.InsState("MC1"+ToStringInteger(j)+"("+lstr+")");
        nextState2 = mouseCounter2.InsState("MC2"+ToStringInteger(j)+"("+lstr+")");
        // intermediate level: mice first come from above and can pass to the lower level
        if(l < n && l > 1){
          event = mouseCounter1.InsControllableEvent("m"+ToStringInteger(l+1)+"-"+lstr );
          mouseCounter1.SetTransition(currentState1,event,nextState1);   
                       
          event = mouseCounter1.InsControllableEvent("m"+lstr+"-"+ToStringInteger(l+1) );
          mouseCounter1.SetTransition(nextState1,event,currentState1);   
                       
          event = mouseCounter2.InsControllableEvent("m"+lstr+"-"+ToStringInteger(l-1) );
          mouseCounter2.SetTransition(currentState2,event,nextState2);  
                            
          event = mouseCounter2.InsControllableEvent("m"+ToStringInteger(l-1)+"-"+lstr );
          mouseCounter2.SetTransition(nextState2,event,currentState2);  
        }  
        // highest level. Initially, all mice are present
        if(l==n){
          if( l > 1){
        	event = mouseCounter1.InsControllableEvent("m"+ToStringInteger(l-1)+"-"+lstr );
        	mouseCounter1.SetTransition(nextState1,event,currentState1);  
                               
        	event = mouseCounter1.InsControllableEvent("m"+lstr+"-"+ToStringInteger(l-1) );
        	mouseCounter1.SetTransition(currentState1,event,nextState1);    
          }     
          else{
    	break;                 
          }                 
        }                
        // first level: mice cannot go to lower levels                               
        if(l == 1 && l != n){
          if(n> 1){
        	event = mouseCounter1.InsControllableEvent("m"+ToStringInteger(l+1)+"-"+lstr );
        	mouseCounter1.SetTransition(currentState1,event,nextState1);  
                               
        	event = mouseCounter1.InsControllableEvent("m"+lstr+"-"+ToStringInteger(l+1) );
        	mouseCounter1.SetTransition(nextState1,event,currentState1);    
          }    
          else{       
    	break;           
          }  
        }
        currentState1 = nextState1;
        currentState2 = nextState2;
      }
      if( l != n && l != 1){
        cParallel(mouseCounter1,mouseCounter2,mouseCounter);   
      }
      else
        mouseCounter = mouseCounter1;
    
      #ifdef FAUDES_DEBUG_WODES08
            mouseCounter.Write("./data/mouseCounter.gen");
      #endif
           
      // Computation of the overall Maze and Maze specification 
      cGenerator cat, mouse;
      // compute the cat
      cParallel(catCounter,catRoom0,cat);
      cParallel(cat,catRoom3,tmp);
      cParallel(tmp,catRoom4,cat);
      cParallel(cat,catRoom1,tmp);
      cParallel(tmp,catRoom2,cat);
      tmp.Clear();
      // compute the mouse
      cParallel(mouseCounter,mouseRoom0,mouse);
      cParallel(mouse,mouseRoom1,tmp);
      cParallel(tmp,mouseRoom2,mouse);
      cParallel(mouse,mouseRoom4,tmp);
      cParallel(tmp,mouseRoom3,mouse);
      tmp.Clear();
      // compute the maze
      cParallel(cat,mouse,rMaze);
      // spec computation
      cParallel(catCounter,mouseCounter,specTmp);
      cParallel(specTmp,specRoom0,rMazeSpec);
      cParallel(rMazeSpec,specRoom1,specTmp);
      cParallel(specTmp,specRoom2,rMazeSpec);
      cParallel(rMazeSpec,specRoom3,specTmp);
      cParallel(specTmp,specRoom4,rMazeSpec);
    
      rMazeSpec.Name("mazeSpec-"+ ToStringInteger(n)+"-"+ToStringInteger(k)+"-"+ToStringInteger(l));
      rMaze.Name("mazePlant-"+ ToStringInteger(n)+"-"+ToStringInteger(k)+"-"+ToStringInteger(l));
    
      #ifdef FAUDES_DEBUG_WODES08
            rMaze.Write("./data/Maze.gen");    
            rMazeSpec.Write("./data/MazeSpec.gen");
      #endif
    
}

