/** @file indexset.cpp @brief Classes IndexSet */

/* FAU Discrete Event Systems Library (libfaudes)

   Copyright (C) 2006  Bernd Opitz
   Copyright (C) 2007  Thomas Moor
   Exclusive copyright is granted to Klaus Schmidt

   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
   License as published by the Free Software Foundation; either
   version 2.1 of the License, or (at your option) any later version.

   This library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Lesser General Public License for more details.

   You should have received a copy of the GNU Lesser General Public
   License along with this library; if not, write to the Free Software
   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA */

#include "indexset.h"


namespace faudes {

// IndexSet()
IndexSet::IndexSet(void) : 
  TBaseSet<Idx>() 
{
  FD_DC("IndexSet(" << this << ")::IndexSet()");
  Name("IndexSet");
}

// IndexSet(rOtherSet)
IndexSet::IndexSet(const TBaseSet<Idx>& rOtherSet) : 
  TBaseSet<Idx>(rOtherSet) 
{
  FD_DC("IndexSet(" << this << ")::IndexSet(rOtherSet " << &rOtherSet << ")");
}

// File constructor
IndexSet::IndexSet(const std::string& rFilename, const std::string& rLabel) :
  TBaseSet<Idx>() 
{
  FD_DC("IndexSet(" << this << ")::IndexSet(" << rFilename << ")");
  Read(rFilename, rLabel);
}

// DoWrite(rTw&)
void IndexSet::DoWrite(TokenWriter& rTw, const std::string& rLabel) const {
  TaIndexSet<AttributeVoid> vcopy= *this;
  vcopy.Write(rTw, rLabel);
}

// DoRead(rTr, rLabel)
void IndexSet::DoRead(TokenReader& rTr, const std::string& rLabel) {
  TaIndexSet<AttributeVoid> vcopy;
  vcopy.Read(rTr,rLabel);
  *this=vcopy;
}

// MaxIndex()
Idx IndexSet::MaxIndex(void) const {
  if(Size()==0) return 0;
  Iterator it= End();
  it--;
  return *it;
}

// Insert()
Idx IndexSet::Insert(void) {
  FD_DC("IndexSet(" << this << ")::Insert()");
  Idx index=MaxIndex()+1;
  TBaseSet<Idx>::Insert(index);
  return index;
}

//Insert(idx)
bool IndexSet::Insert(Idx idx) {
  return TBaseSet<Idx>::Insert(idx);
}


//Valid(idx)
bool  IndexSet::Valid(Idx idx) const {
  return idx!=0;
}


// Signature()
Idx IndexSet::Signature(void) const {
  // prepare result
  Idx sig = 0;
  // helpers:
  Idx i = 1;
  Iterator it;
  // algorithm:
  for (it = Begin(); it != End(); ++it) {
    sig += *it * i;
    ++i;
  }
  return sig;
}



} // end name space
