/* lua_consolehelper.cpp - primitive logging functionality */

/* luafaudes - FAUDES Bindings for Lua

   Copyright (C) 2006 Bernd Opitz
   Exclusive copyright is granted to the Lehrstuhl fuer Regelungstechnik
   at the University of Erlangen-Nuernberg, Germany

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License
   as published by the Free Software Foundation; either version 2
   of the License, or (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA. */



#include "lua_consolehelper.hpp"

std::ofstream* logstream = NULL;
std::string lastline;

int lua_initlog(lua_State* L) {
	lastline = ""; // do not write this as a log entry 
	const char* filename = luaL_checkstring(L, 1);
	if (logstream != NULL) {
		logstream->close();
		delete(logstream);
	}
	if (lua_isstring(L, 2)) {
		const char* str = luaL_checkstring(L, 2);
		std::string openmode(str);
		if (openmode.compare("w")) {
			logstream = new std::ofstream(filename, std::ios::out|std::ios::trunc);
		}
		else if (openmode.compare("a")) {
			logstream = new std::ofstream(filename, std::ios::out|std::ios::app);
		}
		else {
			luaL_error(L, "wrong openmode specified");
		}
	}
	else {
		logstream = new std::ofstream(filename, std::ios::out|std::ios::trunc);
	}
	return 0;
}


void lua_logwrite(const char* str) {
  if(logstream != NULL)
	*logstream << str << std::endl;
}

void lua_logwrite(const std::string& str) {
  if(logstream != NULL)
	*logstream << str << std::endl;
}

int lua_closelog(lua_State* L) {
	if (logstream != NULL) {
		logstream->close();
		delete(logstream);
		logstream = NULL;
	}
	else {
		luaL_error(L, "close of uninitialized logfile");
	}
	return 0;
}

void RegisterConsoleFunctions(lua_State* L) {
	// register functions:
	lua_register(L, "InitLog", lua_initlog);
	lua_register(L, "CloseLog", lua_closelog);
}


void luafaudes_print_version (void) {
	std::stringstream sstr;
	sstr 
		<< "Welcome to luafaudes console." << std::endl
                << "Version: " << FDVersionString() << " / " << FDPluginsString() << " / " << LUA_VERSION << std::endl
	        << "Type 'Help()' for a list of commands.\n";
        fprintf(stderr, "%s\n", sstr.str().c_str());
        fflush(stderr);
}


