/* luafaudes_eset_class.hpp - Lua bindings for the EventSet class */

/* luafaudes - FAUDES Bindings for Lua

   Copyright (C) 2006 Bernd Opitz
   Exclusive copyright is granted to the Lehrstuhl fuer Regelungstechnik
   at the University of Erlangen-Nuernberg, Germany

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License
   as published by the Free Software Foundation; either version 2
   of the License, or (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA. */



#ifndef LUAFAUDES_ESET_CLASS_H
#define LUAFAUDES_ESET_CLASS_H

#include "luafaudes_common.hpp"


// set faudes eventset we support
typedef faudes::cEventSet fEventSet;
#define LUA_EVENTSET "EventSet"



/////////////////////////////////////
// bindings for the EventSet class
/////////////////////////////////////

/**
 * Return EventSet* from lua stack
 *
 * @param L
 *   lua_State*
 * @param index
 *   Index on lua stack
 *
 * @return
 *   fEventSet*
 */
fEventSet* lua_toeventset(lua_State *L, int index);

/**
 * Test whether index on the stack containts a EventSet* userdatum
 * and if the pointer contains a valid EventSet. 
 * 
 * @param L
 *   lua_State*
 * @param index
 *   Index on lua stack
 *
 * @return
 *   fEventSet*; NULL if there is no valid EventSet*
 */
fEventSet* luaL_checkeventset(lua_State *L, int index);


/**
 * Test whether index on the stack containts an EventSet* userdatum
 * 
 * @param L
 *   lua_State*
 * @param index
 *   Index on lua stack
 *
 * @return
 *   fGenerator*; NULL if there is no valid fEventSet*
 */
bool luaL_iseventset(lua_State* L, int index);

/**
 * Create a new userdata of type LUA_EVENTSET. In the c++ context this 
 * creates a new EventSet on the heap and assigns the pointer as the lua 
 * userdatum. Returns the EventSet*.
 *
 * @param L
 *   lua_State*
 *
 * @return
 *   fEventSet*
 */
fEventSet* lua_pusheventset(lua_State *L);

/**
 * (lua user function)
 *
 * Creates new EventSet with pushEventSet
 *
 * @param L
 *   lua_State*
 *
 * @return
 *   Number of new values on the virtual stack
 */
int EventSet_New(lua_State *L);

/**
 * Garbage collection for EventSet userdata values. Calls
 * the delete operator on the EventSet.
 *
 * @param L
 *   lua_State*
 *
 * @return
 *   Number of new values on the virtual stack
 */
int EventSet__gc(lua_State *L);

/**
 * (lua user function)
 *
 * (c-) String conversion method for EventSets
 *
 * @param L
 *   lua_State*
 *
 * @return
 *   Number of new values on the virtual stack
 */
int EventSet__tostring (lua_State *L);



///////////////////////////
// EventSet methods
///////////////////////////

int EventSet_Empty(lua_State* L);

int EventSet_Name(lua_State* L);

int EventSet_Size(lua_State* L);

int EventSet_Write(lua_State *L);

int EventSet_DWrite(lua_State *L);

int EventSet_ToString(lua_State* L);

int EventSet_Read(lua_State *L);

int EventSet_Insert(lua_State* L);

int EventSet_Erase(lua_State* L);

int EventSet_Clear(lua_State* L);

int EventSet_SymbolicName(lua_State* L);

int EventSet_Index(lua_State* L);

int EventSet_Exists(lua_State* L);

//////////////////////////////////
// EventSet methods registration
//////////////////////////////////

/**
 * Register LUA_EVENTSET
 *
 * @param L
 *   lua_State*
 *
 * @return
 *   Number of new values on the virtual stack
 */
int RegisterEventSetClass(lua_State *L);


#endif
